/*
 * This file is part of Crystal Dock.
 * Copyright (C) 2022 Viet Dang (dangvd@gmail.com)
 *
 * Crystal Dock is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Crystal Dock is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Crystal Dock.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "color_button.h"

#include <QColorDialog>
#include <QIcon>

namespace crystaldock {

ColorButton::ColorButton(QWidget *parent) : QPushButton(parent) {
  setAutoFillBackground(true);
  connect(this, SIGNAL(clicked()), this, SLOT(showColorDialog()));
}

void ColorButton::setColor(const QColor& color) {
  color_ = color;
  QPalette pal = palette();
  pal.setColor(QPalette::Button, color);
  setPalette(pal);
  update();
}

void ColorButton::showColorDialog() {
  QColor newColor = QColorDialog::getColor(color_);
  if (newColor.isValid()) {
    setColor(newColor);
  }
}

}  // namespace crystaldock
